
class SignalPhrase:
    def __init__(self):
        self.language = None
        self.tempName = None
        self.tempStr = None
        self.tempToNL = None
        self.inclinations = None
        self.negate = False
        self.force = False
        self.edge = False
        self.vars = {}
        self.values = {}
        self.niceStr = None